@@ -26,10 +26,12 @@
 
 #include <QCoreApplication>
 #include <QDir>
-#include <QDomDocument>
 #include <QMatrix>
 #include <QPainter>
 #include <QStringBuilder>
+#include <QXmlStreamReader>
+#include <QXmlStreamWriter>
+#include <QBuffer>
 
 #include <kcolorscheme.h>
 #include <kconfiggroup.h>
@@ -84,29 +86,29 @@ bool SharedSvgRenderer::load(
 {
     // Apply the style sheet.
     if (!styleSheet.isEmpty() && contents.contains("current-color-scheme")) {
-        QDomDocument svg;
-        if (!svg.setContent(contents)) {
-            return false;
-        }
+        QByteArray processedContents;
+        QXmlStreamReader reader(contents);
 
-        QDomNode defs = svg.elementsByTagName(QStringLiteral("defs")).item(0);
-
-        const QString STYLE = QStringLiteral("style");
-        for (QDomElement style = defs.firstChildElement(STYLE); !style.isNull();
-                style = style.nextSiblingElement(STYLE)) {
-            if (style.attribute(QStringLiteral("id")) == QLatin1String("current-color-scheme")) {
-                QDomElement colorScheme = svg.createElement(STYLE);
-                colorScheme.setAttribute(QStringLiteral("type"), QStringLiteral("text/css"));
-                colorScheme.setAttribute(QStringLiteral("id"), QStringLiteral("current-color-scheme"));
-                defs.replaceChild(colorScheme, style);
-                colorScheme.appendChild(svg.createCDATASection(styleSheet));
-
-                interestingElements.insert(QStringLiteral("current-color-scheme"), QRect(0, 0, 1, 1));
-
-                break;
+        QBuffer buffer(&processedContents);
+        buffer.open(QIODevice::WriteOnly);
+        QXmlStreamWriter writer(&buffer);
+        while (!reader.atEnd()) {
+            if (reader.readNext() == QXmlStreamReader::StartElement &&
+                reader.qualifiedName() == QLatin1String("style") &&
+                reader.attributes().value(QLatin1String("id")) == QLatin1String("current-color-scheme")) {
+                writer.writeStartElement(QLatin1String("style"));
+                writer.writeAttributes(reader.attributes());
+                writer.writeCharacters(styleSheet);
+                writer.writeEndElement();
+                while (reader.tokenType() != QXmlStreamReader::EndElement) {
+                    reader.readNext();
+                }
+            } else if (reader.tokenType() != QXmlStreamReader::Invalid) {
+                writer.writeCurrentToken(reader);
             }
         }
-        if (!QSvgRenderer::load(svg.toByteArray(-1))) {
+        buffer.close();
+        if (!QSvgRenderer::load(processedContents)) {
             return false;
         }
     } else if (!QSvgRenderer::load(contents)) {
@@ -134,8 +136,8 @@ bool SharedSvgRenderer::load(
 }
 
 #define QLSEP QLatin1Char('_')
-#define CACHE_ID_WITH_SIZE(size, id, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
-#define CACHE_ID_NATURAL_SIZE(id, devicePixelRatio) QLatin1String("Natural") % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
+#define CACHE_ID_WITH_SIZE(size, id, status, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QString::number(status) % QLSEP % QString::number(int(devicePixelRatio))
+#define CACHE_ID_NATURAL_SIZE(id, status, devicePixelRatio) QLatin1String("Natural") % QLSEP % id % QLSEP % QString::number(status) % QLSEP % QString::number(int(devicePixelRatio))
 
 SvgPrivate::SvgPrivate(Svg *svg)
     : q(svg),
@@ -145,6 +147,7 @@ SvgPrivate::SvgPrivate(Svg *svg)
       lastModified(0),
       devicePixelRatio(1.0),
       scaleFactor(1.0),
+      status(Svg::Status::Normal),
       multipleImages(false),
       themed(false),
       useSystemColors(false),
@@ -165,16 +168,16 @@ SvgPrivate::~SvgPrivate()
 QString SvgPrivate::cacheId(const QString &elementId) const
 {
     if (size.isValid() && size != naturalSize) {
-        return CACHE_ID_WITH_SIZE(size, elementId, devicePixelRatio);
+        return CACHE_ID_WITH_SIZE(size, elementId, status, devicePixelRatio);
     } else {
-        return CACHE_ID_NATURAL_SIZE(elementId, devicePixelRatio);
+        return CACHE_ID_NATURAL_SIZE(elementId, status, devicePixelRatio);
     }
 }
 
 //This function is meant for the pixmap cache
 QString SvgPrivate::cachePath(const QString &path, const QSize &size) const
 {
-    return CACHE_ID_WITH_SIZE(size, path, devicePixelRatio) % QLSEP % QString::number(colorGroup);
+    return CACHE_ID_WITH_SIZE(size, path, status, devicePixelRatio) % QLSEP % QString::number(colorGroup);
 }
 
 bool SvgPrivate::setImagePath(const QString &imagePath)
@@ -306,7 +309,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
     if (elementsWithSizeHints.isEmpty()) {
         // Fetch all size hinted element ids from the theme's rect cache
         // and store them locally.
-        QRegExp sizeHintedKeyExpr(CACHE_ID_NATURAL_SIZE("(\\d+)-(\\d+)-(.+)", ratio));
+        QRegExp sizeHintedKeyExpr(CACHE_ID_NATURAL_SIZE("(\\d+)-(\\d+)-(.+)", status, ratio));
 
         foreach (const QString &key, cacheAndColorsTheme()->listCachedRectKeys(path)) {
             if (sizeHintedKeyExpr.exactMatch(key)) {
@@ -452,7 +455,7 @@ void SvgPrivate::createRenderer()
     //qCDebug(LOG_PLASMA) << "FAIL! **************************";
     //qCDebug(LOG_PLASMA) << path << "**";
 
-    QString styleSheet = cacheAndColorsTheme()->d->svgStyleSheet(colorGroup);
+    QString styleSheet = cacheAndColorsTheme()->d->svgStyleSheet(colorGroup, status);
     styleCrc = qChecksum(styleSheet.toUtf8(), styleSheet.size());
 
     QHash<QString, SharedSvgRenderer::Ptr>::const_iterator it = s_renderers.constFind(styleCrc + path);
@@ -475,7 +478,7 @@ void SvgPrivate::createRenderer()
                 const QString &elementId = i.key();
                 const QRectF &elementRect = i.value();
 
-                const QString cacheId = CACHE_ID_NATURAL_SIZE(elementId, devicePixelRatio);
+                const QString cacheId = CACHE_ID_NATURAL_SIZE(elementId, status, devicePixelRatio);
                 localRectCache.insert(cacheId, elementRect);
                 cacheAndColorsTheme()->insertIntoRectsCache(path, cacheId, elementRect);
             }
@@ -977,6 +980,23 @@ Theme *Svg::theme() const
     return d->actualTheme();
 }
 
+void Svg::setStatus(Plasma::Svg::Status status)
+{
+    if (status == d->status) {
+        return;
+    }
+
+    d->status = status;
+    d->eraseRenderer();
+    emit statusChanged(status);
+    emit repaintNeeded();
+}
+
+Svg::Status Svg::status() const
+{
+    return d->status;
+}
+
 } // Plasma namespace
 
 #include "private/moc_svg_p.cpp"
